--iTEAM - GISED
--Created:
--Revised:

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all;
use work.lyp6x32r64z_pkg.all;

entity lyp6x32r64z is
	port(id_llr_block : in p_type_a2048xQIstd;           --llr block : qi
	     ic_decode : in std_logic;                       --decode enable signal
	     od_dec_block : out std_logic_vector(0 to 2047); --decoded block : s[1 0]
	     od_dloop : out std_logic_vector(7 downto 0);    --actual iteration : u[8 0]
	     od_parity : out std_logic;                      --parity check
	     oc_dec_rdy : out std_logic;                     --decode ready signal
	     clk : in std_logic;                             --clock signal
	     rst : in std_logic);                            --reset signal
end lyp6x32r64z;

architecture rtl of lyp6x32r64z is
	--type type_a6x6std is array(0 to 5) of std_logic_vector(5 downto 0);
	--type type_a7x6std is array(0 to 6) of std_logic_vector(5 downto 0);
	--type type_a32xQIstd is array(0 to 31) of std_logic_vector(qi-1 downto 0);
	--type type_a64x32ind is array(0 to 63) of std_logic_vector(0 to 31);
	--type type_a192xQLstd is array(0 to 191) of std_logic_vector(ql-1 downto 0);
	--type type_a192xQMstd is array(0 to 191) of std_logic_vector(qm-1 downto 0);
	--type type_a256x6ind is array(0 to 255) of std_logic_vector(0 to 5);
	type type_a2048xQLstd is array(0 to 2047) of std_logic_vector(ql-1 downto 0);
	--components declaration
	component lyp6x32r64z_lnmux is
		port(id_lnm0 : in p_type_a32xQLstd;             --lambda(n) layer 0 : ql
		     id_lnm1 : in p_type_a32xQLstd;             --lambda(n) layer 1 : ql
		     id_lnm2 : in p_type_a32xQLstd;             --lambda(n) layer 2 : ql
		     id_lnm3 : in p_type_a32xQLstd;             --lambda(n) layer 3 : ql
		     id_lnm4 : in p_type_a32xQLstd;             --lambda(n) layer 4 : ql
		     id_lnm5 : in p_type_a32xQLstd;             --lambda(n) layer 5 : ql
		     id_llrn : in p_type_a32xQIstd;             --llr(n) : qi
		     ic_seli : in std_logic_vector(2 downto 0); --u[3 0] : layers 0 to 5, llr
		     od_lnsel : out p_type_a32xQLstd);          --lambda(n) selected : ql
	end component;
	component lyp6x32r64z_pnu32i is
		generic(latency : integer);                   --clock cycles latency : 1 to 1
		port(id_lnold : in p_type_a32xQLstd;          --lambda old(n) : ql
		     ic_init : in std_logic;                  --initialize control signal
			  id_dloop : in std_logic_vector(4 downto 0);-- 
		     od_decn : out std_logic_vector(0 to 31); --dec(n) : s[1 0]
			  od_parm : out std_logic;					 		--parity check
		     od_lnnew : out p_type_a32xQLstd;         --lambda new(n) : ql
		     clk : in std_logic);                     --clock signal
	end component;
	--b0: general control
	signal b0_subit_count_s,b0_subit_count_r: std_logic_vector(2 downto 0); --u[3 0]
	signal b0_pnu_seli_s,b0_pnu_seli_r: std_logic_vector(2 downto 0); --u[3 0]
	signal b0_end_loop_s: std_logic;
	signal b0_loop_count_r: std_logic_vector(7 downto 0); --u[8 0]
	signal b0_pnu_ctl_r: std_logic;
	--b1: processor node units
	signal b1_decn_r: std_logic_vector(0 to 2047);
	signal b1_lambdan_ra: type_a2048xQLstd; --ql
	signal b1_parm_block_r: std_logic_vector(0 to 63);
	--b2: decoded data and parity management
	signal b2_dec_block_s: std_logic_vector(0 to 2047);
	signal b2_dloop_r: std_logic_vector(7 downto 0); --u[8 0]
	signal b2_par_block_r: std_logic_vector(0 to 4);
	signal b2_par_block_s: std_logic;
	signal b2_parity_s,b2_parity_r: std_logic;
	signal b2_dec_rdy_r: std_logic;
	--
begin
	--b0: general control
	process(clk)
	begin
		if rising_edge(clk) then
			if rst='1' then
				b0_subit_count_r <= "000";
				b0_pnu_seli_r <= "000";
				b0_loop_count_r <= "00000000";
				b0_pnu_ctl_r <= '0';
			else
				if b0_subit_count_r="101" or ic_decode='0' then
					b0_subit_count_r <= "000";
					b0_pnu_seli_r <= b0_pnu_seli_s;
				else
					b0_subit_count_r <= b0_subit_count_s;
					b0_pnu_seli_r <= b0_subit_count_s;
				end if;
				if ic_decode='0' then
					b0_loop_count_r <= "00000000";
					b0_pnu_ctl_r <= '0';
				else
					b0_loop_count_r <= b0_loop_count_r + b0_end_loop_s;
					if b0_end_loop_s='1' then
						b0_pnu_ctl_r <= '1';
					end if;
				end if;
			end if;
		end if;
	end process;
	b0_subit_count_s <= b0_subit_count_r + '1';
	b0_pnu_seli_s <= "111" when ic_decode='0' else "000";
	b0_end_loop_s <= '1' when b0_subit_count_r="101" else '0';
	--b1: processor node units
	GPNU: for eP in 0 to 63 generate
		signal g_lnm0_mux_sa: p_type_a32xQLstd;
		signal g_lnm1_mux_sa: p_type_a32xQLstd;
		signal g_lnm2_mux_sa: p_type_a32xQLstd;
		signal g_lnm3_mux_sa: p_type_a32xQLstd;
		signal g_lnm4_mux_sa: p_type_a32xQLstd;
		signal g_lnm5_mux_sa: p_type_a32xQLstd;
		signal g_lnold_pnu_sa: p_type_a32xQLstd;
		signal g_llrn_pnu_sa: p_type_a32xQIstd;
		signal g_decn_r: std_logic_vector(0 to 31);
		signal g_lnnew_pnu_ra: p_type_a32xQLstd;
	begin
		CM: lyp6x32r64z_lnmux port map(id_lnm0 => g_lnm0_mux_sa,
		                               id_lnm1 => g_lnm1_mux_sa,
		                               id_lnm2 => g_lnm2_mux_sa,
		                               id_lnm3 => g_lnm3_mux_sa,
		                               id_lnm4 => g_lnm4_mux_sa,
		                               id_lnm5 => g_lnm5_mux_sa,
		                               id_llrn => g_llrn_pnu_sa,
		                               ic_seli => b0_pnu_seli_r,
		                               od_lnsel => g_lnold_pnu_sa);
		CP: lyp6x32r64z_pnu32i generic map(latency => lpnu)
		                       port map(id_lnold => g_lnold_pnu_sa,
		                                ic_init => b0_pnu_ctl_r,
												  id_dloop => b0_loop_count_r(4 downto 0),
		                                od_decn => g_decn_r,
												  od_parm => b1_parm_block_r(eP),
		                                od_lnnew => g_lnnew_pnu_ra,
		                                clk => clk);
		GS: for eS in 0 to 31 generate
			g_lnm0_mux_sa(eS) <= b1_lambdan_ra(p_idlnm0_cm(eP)(eS));
			g_lnm1_mux_sa(eS) <= b1_lambdan_ra(p_idlnm1_cm(eP)(eS));
			g_lnm2_mux_sa(eS) <= b1_lambdan_ra(p_idlnm2_cm(eP)(eS));
			g_lnm3_mux_sa(eS) <= b1_lambdan_ra(p_idlnm3_cm(eP)(eS));
			g_lnm4_mux_sa(eS) <= b1_lambdan_ra(p_idlnm4_cm(eP)(eS));
			g_lnm5_mux_sa(eS) <= b1_lambdan_ra(p_idlnm5_cm(eP)(eS));
			g_llrn_pnu_sa(eS) <= id_llr_block(p_idllrn_cm(eP)(eS));
			b1_decn_r(64*eS+eP) <= g_decn_r(eS);
			b1_lambdan_ra(64*eS+eP) <= g_lnnew_pnu_ra(eS);
		end generate;
	end generate;
	--b2: decoded data and parity management
	process(clk)
	begin
		if rising_edge(clk) then
			if rst='1' then
				b2_dloop_r <= "00000000";
				b2_par_block_r <= (others => '0');
				b2_parity_r <= '0';
				b2_dec_rdy_r <= '0';
			else
				if b0_end_loop_s='1' then
					b2_dloop_r <= b0_loop_count_r;
				end if;
				b2_par_block_r <= b2_par_block_s&b2_par_block_r(0 to 3);
				b2_parity_r <= b2_parity_s;
				b2_dec_rdy_r <= b0_end_loop_s;
			end if;
		end if;
	end process;
	b2_dec_block_s <= b1_decn_r;
	b2_par_block_s <= '1' when b1_parm_block_r=X"FFFFFFFFFFFFFFFF" else '0';
	b2_parity_s <= b2_par_block_s when b2_par_block_r="11111" else '0';
	--
	od_dec_block <= b2_dec_block_s;
	od_dloop <= b2_dloop_r;
	od_parity <= b2_parity_r;
	oc_dec_rdy <= b2_dec_rdy_r;
end rtl;
