--iTEAM - GISED
--Created:
--Revised:

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all;
use work.lyp6x32r64z_pkg.all;

entity lyp6x32r64z_lnmux is
	port(id_lnm0 : in p_type_a32xQLstd;             --lambda(n) layer 0 : ql
	     id_lnm1 : in p_type_a32xQLstd;             --lambda(n) layer 1 : ql
	     id_lnm2 : in p_type_a32xQLstd;             --lambda(n) layer 2 : ql
	     id_lnm3 : in p_type_a32xQLstd;             --lambda(n) layer 3 : ql
	     id_lnm4 : in p_type_a32xQLstd;             --lambda(n) layer 4 : ql
	     id_lnm5 : in p_type_a32xQLstd;             --lambda(n) layer 5 : ql
	     id_llrn : in p_type_a32xQIstd;             --llr(n) : qi
	     ic_seli : in std_logic_vector(2 downto 0); --u[3 0] : layers 0 to 5, llr
	     od_lnsel : out p_type_a32xQLstd);          --lambda(n) selected : ql
end lyp6x32r64z_lnmux;

architecture rtl of lyp6x32r64z_lnmux is
	type type_a32xQLstd is array(0 to 31) of std_logic_vector(ql-1 downto 0);
	--b0: double-port ram
	signal b0_lnsel_s: type_a32xQLstd;
	--
begin
	--b0: multiplexor
	process(id_lnm0,id_lnm1,id_lnm2,id_lnm3,id_lnm4,id_lnm5,id_llrn,ic_seli)
	begin
		for iM in 0 to 31 loop
			case ic_seli is
				when "000" => b0_lnsel_s(iM) <= id_lnm0(iM);
				when "001" => b0_lnsel_s(iM) <= id_lnm1(iM);
				when "010" => b0_lnsel_s(iM) <= id_lnm2(iM);
				when "011" => b0_lnsel_s(iM) <= id_lnm3(iM);
				when "100" => b0_lnsel_s(iM) <= id_lnm4(iM);
				when "101" => b0_lnsel_s(iM) <= id_lnm5(iM);
				when "110" => b0_lnsel_s(iM) <= (others => '0');
				when "111" => b0_lnsel_s(iM) <= sxt(id_llrn(iM),ql);
				when others => b0_lnsel_s(iM) <= (others => '0');
			end case;
		end loop;
	end process;
	--
	process(b0_lnsel_s)
	begin
		for iC in 0 to 31 loop
			od_lnsel(iC) <= b0_lnsel_s(iC);
		end loop;
	end process;
end rtl;
