--iTEAM - GISED
--Created:
--Revised:

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.lyp6x32r64z_pkg.all;

entity lyp6x32r64z_pnu32i is
	generic(latency : integer:=1);                  --clock cycles latency : 1 to 1
	port(id_lnold : in p_type_a32xQLstd;          	--lambda old(n) : ql
	     ic_init : in std_logic;                  	--initialize control signal
		  id_dloop : in std_logic_vector(4 downto 0);--actual iteration : u[8 0] 
	     od_decn : out std_logic_vector(0 to 31); 	--dec(n) : s[1 0]
		  od_parm : out std_logic;					 		--parity check
	     od_lnnew : out p_type_a32xQLstd;         	--lambda new(n) : ql
	     clk : in std_logic);                     	--clock signal
end lyp6x32r64z_pnu32i;

architecture rtl of lyp6x32r64z_pnu32i is
	-- En_calc_ROM
	component lyp6x32r64z_En_calc_ROM is
    Port (ic_address: in  std_logic_vector(1 downto 0);
          od_En_calc : out  std_logic_vector(3 downto 0));
	end component;
--	component shift_register_mWxnB is
--	 Generic (mW: integer:=6;	-- Width
--				 nB: integer:=4);	-- Bits
--    Port (id_SR: in  std_logic_vector(nB-1 downto 0);	 -- Input data
--          od_SR : out  std_logic_vector(nB-1 downto 0); -- Output data 
--			 clk : in std_logic);    -- clock signal
--	end component;
	-- types
	type type_a33x30std is array (0 to 32) of std_logic_vector (0 to 30);
	type type_a32x2std is array(0 to 31) of std_logic_vector(1 downto 0);
	type type_a32x4std is array(0 to 31) of std_logic_vector(3 downto 0);
	type type_a32x7std is array(0 to 31) of std_logic_vector(6 downto 0);
	type memory_thresholdBF is array(0 to 17) of std_logic_vector(5 downto 0);-- s[6 1]
	signal ld_min1 : type_a33x30std;
	signal ld_sm : std_logic_vector (0 to 30); -- (Sm(k-2),S(k-1),...,Sm(0))
	signal ld_parm : std_logic_vector (0 to 30); -- (Sm(k-2),S(k-1),...,Sm(0))
	signal ld_min2,ld_wmn: std_logic_vector (0 to 32-1); -- 
	--b0: processor-node update
	signal b0_lnnew_ra: p_type_a32xQLstd;
	signal mem_thresholds: memory_thresholdBF := ("100000","100000","100000","011110","011110","011110","011101",
	"011101","011011","011011","011011","011010","011010","011010","011001","011001","011001","011000");-- u[6 2]
	--
	signal ld_zn_new: std_logic_vector(31 downto 0);
	signal b0_address_En : type_a32x2std;-- u[2 0]
	signal b0_En_calc_out: type_a32x4std;-- u[4 1]
	signal b0_En_calc_total,b0_En_calc_cntr: type_a32x7std;-- u[6 2]
	signal b0_En_calc_cntr_div: type_a32x7std;-- u[6 3]
	signal ld_En_sr_out : type_a32x4std;
--	signal ld_En_mux_init_sa : type_a32x4std;
	signal b0_parm_r : std_logic;
	signal ld_threshold: std_logic_vector(5 downto 0);
	
begin
	-- Syndr calc (id_lnold(i)(1) = id_zn(i))
	process (id_lnold,ld_sm)
	begin 
		ld_sm (0) <=  id_lnold(0)(1) xor id_lnold(1)(1);	
		xor_tree:for i in 0 to 29 loop
				ld_sm(i+1) <= ld_sm(i) xor id_lnold(i+2)(1);
		end loop;
	end process;
	-- Min calc (id_lnold(j)(0) = id_yr(j))
	process (id_lnold,ld_min1)
	begin 
		for j in 0 to 32 loop
			if j=1 then
				ld_min1(j)(0) <= (not id_lnold(0)(0)) and id_lnold(1)(0);
			elsif j=2 then
				ld_min1(j)(0) <= id_lnold(0)(0) and (not id_lnold(1)(0));
			else
				ld_min1(j)(0) <= id_lnold(0)(0) and id_lnold(1)(0);
			end if;
			for i in 0 to 29 loop
				if j<3 then
					ld_min1(j)(i+1) <= ld_min1(j)(i) and id_lnold(i+2)(0);
				else
					if i=(j-3) then
						ld_min1(j)(i+1) <= ld_min1(j)(i) and (not id_lnold(i+2)(0));
					elsif i/=(j-3) then
						ld_min1(j)(i+1) <= ld_min1(j)(i) and id_lnold(i+2)(0);
					end if;
				end if;
			end loop;
		end loop;
	end process;
	-- Out min2
	process (ld_min1,ld_min2)
	begin 
		ld_min2(0) <= ld_min1(0)(30) or ld_min1(1)(30);
		for i in 0 to 30 loop
			ld_min2 (i+1) <= ld_min2 (i)  or ld_min1(i+2)(30);
		end loop;
	end process;
	-- Fmin function - ld_wmn = yr'·min2 + min1
	process (id_lnold,ld_min1,ld_min2)
	begin 
		for i in 0 to 32-1 loop
			ld_wmn(i) <= ((not id_lnold(i)(0)) and ld_min2(32-1)) or (ld_min1(0)(30));
		end loop;		
	end process;
	--En calc ROMs
	N_ROMs: for iL in 0 to 31 generate
		b0_address_En(iL) <= ld_wmn(iL) & ld_sm(30);
		ECR: lyp6x32r64z_En_calc_ROM port map(ic_address => b0_address_En(iL),
														  od_En_calc => b0_En_calc_out(iL));
	end generate;
	-- Total En calc Adders
	process(b0_En_calc_out,b0_En_calc_cntr_div)
	begin
		for iL in 0 to 31 loop
			b0_En_calc_total(iL) <= ('0'&b0_En_calc_out(iL)&"00") + b0_En_calc_cntr_div(iL);
		end loop;
	end process;
	-- En Shift registers
--	N_ESRs: for iL in 0 to 31 generate
--		ESR: shift_register_mWxnB generic map (6,4)
--										  port map (id_SR => b0_En_calc_out(iL),
--													   od_SR => ld_En_sr_out(iL),
--													   clk => clk);
--	end generate;
	-- En mux init
--	process(ic_init,ld_En_sr_out)
--	begin
--		for iL in 0 to 31 loop
--			if ic_init = '0' then
--				ld_En_mux_init_sa(iL) <= (others=>'0');
--			else
--				ld_En_mux_init_sa (iL) <= ld_En_sr_out(iL);
--			end if;
--		end loop;
--	end process;
	-- En calc contribution
	process(b0_En_calc_cntr,id_lnold) --ld_En_mux_init_sa
	begin
		for iL in 0 to 31 loop
			--b0_En_calc_cntr(iL) <= id_lnold(iL)(8 downto 2) - ('0'&ld_En_mux_init_sa(iL)&"00");
			b0_En_calc_cntr(iL) <= id_lnold(iL)(8 downto 2);
			b0_En_calc_cntr_div(iL) <= "00" & b0_En_calc_cntr(iL)(6 downto 2);
		end loop;
	end process;
	
	-- Thresholds memory
	ld_threshold <= mem_thresholds(conv_integer(unsigned(id_dloop)));
	-- Bit flip 
	process(b0_En_calc_total,ld_threshold,id_lnold)
	begin
		for iL in 0 to 31 loop
			if b0_En_calc_total(iL)(6 downto 1) >= ld_threshold then
				ld_zn_new (iL) <= not id_lnold(iL)(1);
			else
				ld_zn_new (iL) <= id_lnold(iL)(1);
			end if;
		end loop;
	end process;
	-- Parity calc
	process (ld_zn_new,ld_parm)
	begin 
		ld_parm (0) <=  ld_zn_new(0) xor ld_zn_new(1);	
		xor_tree:for i in 0 to 29 loop
				ld_parm(i+1) <= ld_parm(i) xor ld_zn_new(i+2);
		end loop;
	end process;	
	-- Output registers
	process(clk)
	begin
		if rising_edge(clk) then
			for iL in 0 to 31 loop
				b0_lnnew_ra(iL) <= b0_En_calc_total(iL)&ld_zn_new(iL)&id_lnold(iL)(0);
			end loop;
			b0_parm_r <= ld_parm(30);
		end if;
	end process;
	-- Output signals
	od_lnnew <= b0_lnnew_ra;
	od_parm <= b0_parm_r;
	process(b0_lnnew_ra)
	begin
		for iL in 0 to 31 loop
			od_decn(iL) <= b0_lnnew_ra(iL)(1);
		end loop;
	end process;
end rtl;