--iTEAM - GISED
--Created:
--Revised:

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all;
use ieee.std_logic_textio.all;
use std.textio.all;
use work.lyp6x32r64z_pkg.all;

entity lyp6x32r64z_top_tsb is
end lyp6x32r64z_top_tsb;

architecture tb of lyp6x32r64z_top_tsb is
	-- Component declaration of the tested unit
	component lyp6x32r64z_top is
		port(id_yn_block : in std_logic_vector(0 to 1); --yn block 0 : qi
	     ic_decode : in std_logic;                               --decode enable signal
	     od_dec_serial : out std_logic;         						 --decoded block : s[1 0]		  
	     od_dec_block : out std_logic_vector(0 to 2047);         --decoded block : s[1 0]
	     od_dloop : out std_logic_vector(7 downto 0);            --actual iteration : u[8 0]
	     od_parity : out std_logic;                              --parity check
	     oc_dec_rdy : out std_logic;                             --decode ready signal
	     clk : in std_logic;                                     --clock signal
	     rst : in std_logic);                                    --reset signal
	end component;
	-- types
	type type_a32x7std is array(0 to 31) of std_logic_vector(6 downto 0);-- u[7 3]
	type type_a32x2std is array(0 to 31) of std_logic_vector(0 to 1);-- u[7 3]	
	-- Stimulus signals - signals mapped to the input and inout ports of tested entity
	signal id_yn_block: std_logic_vector(0 to 1):= (others => '0');
	signal ic_decode: std_logic:= '0';
	signal clk: std_logic:= '0';
	signal rst: std_logic:= '1';
	-- Observed signals - signals mapped to the output ports of tested entity
	signal od_dec_block: std_logic_vector(0 to 2047):= (others => '0');
	signal od_dloop: std_logic_vector(7 downto 0):= (others => '0');
	signal od_parity: std_logic;
	signal oc_dec_rdy: std_logic;
	-- Test signals
	constant clkP: time:= 20 ns;
	signal enaI: std_logic:= '0';
	signal id_tnu_0_f: type_a32x2std:= (others => (others => '0'));
	signal od_dec_block_f,od_dec_block_c: std_logic_vector(0 to 2047):= (others => '0');
	signal od_En_block_f: type_a32x7std:= (others => (others => '0'));
	signal od_parity_f,od_parity_c: std_logic:= '0';
	signal cnt_dec_block_error, cnt_parity_error: integer:= 0;
	--
	signal end_sim: std_logic:= '0';
begin
	-- Unit Under Test port map
	UUT : lyp6x32r64z_top
	port map(id_yn_block => id_yn_block,
				ic_decode => ic_decode,
				od_dec_block => od_dec_block,
				od_dloop => od_dloop,
				od_parity => od_parity,
				oc_dec_rdy => oc_dec_rdy,
				clk => clk,
				rst => rst);
	-- Stimulus
	clk <= not clk after clkP/2;
	rst <= '0' after clkP*4;
	enaI <= '1' after clkP*8;

	process(clk)
		file f_id: text open read_mode is "./src/stm/t_id_zeros_seed[1985]_EbN0[8.2].txt";
		file f_id_tnu_0: text open read_mode is "./src/stm/t_id_zeros_seed[1985]_EbN0[8.2]_tnu(0).txt";
		file f_od: text open read_mode is "./src/stm/t_od_zeros_seed[1985]_EbN0[8.2].txt";
		file f_od_En: text open read_mode is "./src/stm/t_od_zeros_En_seed[1985]_EbN0[8.2].txt";		
		variable l_id: line;
		variable v_id_0: std_logic_vector(0 to 1);
		variable l_id_tnu_0: line;
		variable v_id_tnu_0: type_a32x2std;
		variable l_od: line;
		variable v_od_0: std_logic_vector(0 to 2047);
		variable v_od_1: std_logic;
		variable l_od_En: line;
		variable v_od_En_0: type_a32x7std;
	begin
		if rising_edge(clk) then
			if enaI='1' then
				if not endfile(f_id) then
					readline(f_id,l_id);
					read(l_id,v_id_0);
					id_yn_block <= v_id_0 after clkP/10;
					ic_decode <= '0';
				else
					id_yn_block <= (others => '0') after clkP/10;
					ic_decode <= '1' after clkP/10;
				end if;
			end if;
			-- ic_decode <= '1' after clkP*(2048+8)+clkP/10;
			if ic_decode='1' then
				if not endfile(f_od) then
					readline(f_od,l_od);
					read(l_od,v_od_0);
					od_dec_block_f <= v_od_0 after clkP/10;
					read(l_od,v_od_1);
					od_parity_f <= v_od_1;
				end if;
				if od_dec_block_f/=od_dec_block_c then 
					cnt_dec_block_error <= cnt_dec_block_error + 1;
				end if;
				if od_parity_f/=od_parity_c then
					cnt_parity_error <= cnt_parity_error + 1;
				end if;
				if not endfile(f_od_En) then
					readline(f_od_En,l_od_En);
					for i in 0 to 31 loop
						read(l_od_En,v_od_En_0(i));
						od_En_block_f(i) <= v_od_En_0(i);
					end loop;
					readline(f_id_tnu_0,l_id_tnu_0);
					for i in 0 to 3 loop
						read(l_id_tnu_0,v_id_tnu_0(i));
						id_tnu_0_f(i) <= v_id_tnu_0(i);
					end loop;
				end if;
			end if;
			if endfile(f_od) then
				end_sim <= '1';
			end if;
		end if;
	end process;
	od_dec_block_c <= od_dec_block after clkP/10;
	od_parity_c <= od_parity after clkP/10;	
	----
	assert end_sim='0'
		report "End of simulation...."
		severity FAILURE;
end tb;
